using System;
using System.Text;
using System.Web.Mvc;

namespace Orchard.Simplex.Utils {

    /// <summary>
    /// Gravatar rating.
    /// </summary>
    public enum Rating {
        /// <summary>
        /// G - suitable for display on all websites with any audience type.
        /// </summary>
        G,
        /// <summary>
        /// PG - may contain rude gestures, provocatively dressed individuals, the lesser swear words, or mild violence.
        /// </summary>
        PG,
        /// <summary>
        /// R - may contain such things as harsh profanity, intense violence, nudity, or hard drug use.
        /// </summary>
        R,
        /// <summary>
        /// X - may contain hardcore sexual imagery or extremely disturbing violence.
        /// </summary>
        X
    }

    /// <summary>
    /// Object that renders Gravatar avatars.
    /// </summary>
    public class Gravatar {

        private static readonly int MinSize = 1;
        private static readonly int MaxSize = 512;

        private int _Size = 80;
        private Rating _MaxRating = Rating.PG;
        private string _PrefixURL = "http://www.gravatar.com/avatar/";

        /// <summary>
        /// The default image to be shown if no Gravatar is found for an email address.
        /// </summary>
        public string DefaultImage { get; set; }

        /// <summary>
        /// The size, in pixels, of the Gravatar to render.
        /// </summary>
        public int Size {
            get { return _Size; }
            set {
                if (value < MinSize || value > MaxSize)
                    throw new ArgumentOutOfRangeException("Size", "The allowable range for 'Size' is '" + MinSize + "' to '" + MaxSize + "', inclusive.");
                _Size = value;
            }
        }

        /// <summary>
        /// The maximum Gravatar rating allowed to display.
        /// </summary>
        public Rating MaxRating {
            get { return _MaxRating; }
            set { _MaxRating = value; }
        }

        public string PrefixURL
        {
            get { return _PrefixURL; }
            set { _PrefixURL = value; }
        }

        /// <summary>
        /// Gets a link to the image file of the Gravatar for the specified <paramref name="email"/>.
        /// </summary>
        /// <param name="email">The email whose Gravatar image source should be returned.</param>
        /// <returns>The URI of the Gravatar for the specified <paramref name="email"/>.</returns>
        public string Init(string email) {
            if (string.IsNullOrEmpty(email) || string.IsNullOrEmpty(email.Trim()))
                throw new ArgumentException("The email is empty.", "email");

            var imageUrl = PrefixURL;
            imageUrl += Hash(email) + ".png";
            imageUrl += "?s=" + Size.ToString();
			imageUrl += "&r=" + MaxRating.ToString();

            var defImg = DefaultImage;
            if (!string.IsNullOrEmpty(defImg))
                imageUrl += "&d=" + System.Web.HttpUtility.UrlEncode(defImg);

            return imageUrl;
        }
		
		private string Hash(string input)
        {
            // Trim the string
            input = input.ToLowerInvariant().Trim();

            // step 1, calculate MD5 hash from input
            var md5 = System.Security.Cryptography.MD5.Create();
            var inputBytes = System.Text.Encoding.UTF8.GetBytes(input);
            var hash = md5.ComputeHash(inputBytes);
            
            // step 2, convert byte array to hex string
            var sb = new StringBuilder();
            for (var i = 0; i < hash.Length; i++)
            {
                sb.Append(hash[i].ToString("X2"));
            }
			
            return sb.ToString().ToLowerInvariant();
        }
    }
	
	/// <summary>
    /// UrlHelper extension methods for retrieving Gravatar avatar images.
    /// </summary>
    public static class UrlHelperExtensions {
    
        /// <summary>
        /// Gets the URI of the Gravatar image for the specifications.
        /// </summary>
        /// <param name="urlHelper">The UrlHelper object getting the URI.</param>
        /// <param name="email">The email whose Gravatar source should be returned.</param>
        /// <param name="size">The size of the requested Gravatar.</param>
        /// <param name="defaultImage">The default image to return if no Gravatar is found for the specified <paramref name="email"/>.</param>
        /// <param name="maxRating">The maximum Gravatar rating to allow for requested images..</param>
        /// <returns>The URI of the Gravatar for the specifications.</returns>
        public static string Gravatar(this UrlHelper urlHelper, string email, int? size, string defaultImage, Rating? maxRating) {
            var gravatar = new Gravatar();
            gravatar.DefaultImage = defaultImage;
            if (size.HasValue) gravatar.Size = size.Value;
            if (maxRating.HasValue) gravatar.MaxRating = maxRating.Value;

            return gravatar.Init(email);
        }

        /// <summary>
        /// Gets the URI of the Gravatar image for the specifications.
        /// </summary>
        /// <param name="urlHelper">The UrlHelper object getting the URI.</param>
        /// <param name="email">The email whose Gravatar source should be returned.</param>
        /// <param name="size">The size of the requested Gravatar.</param>
        /// <returns>The URI of the Gravatar for the specifications.</returns>
        public static string Gravatar(this UrlHelper urlHelper, string email, int? size) {
            return Gravatar(urlHelper, email, size, null, null);
        }
    }
}